/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

/**
 * CeboMsr - C++ example - Frame I/O.
 *
 * Multiple inputs are read multiple times in a single transaction,
 * values are printed to the console.
 * AO-1 and DP-2 are set multiple times in a single transaction.
 *
 * Cebo-Stick example does input transfer only.
 */

#include <iostream>
#include <stdexcept>
#include <cmath>

#include <cebomsrpp.h>

#include <boost/date_time.hpp>

using namespace std;
using namespace CeboMsr;

/**
 * Do frame IO (read/write multiple I/O in single calls).
 */
void frameIoTestLC(Device &device) {
	// Prepare and fill the list of inputs to read.
	InputVector inputs;
	inputs
		<< device.getSingleEndedInputs().at(0)
		<< device.getSingleEndedInputs().at(1)
		<< device.getDifferentialInputs().at(1)
		<< device.getDigitalPorts().at(0)
		<< device.getDigitalPorts().at(1)
		<< device.getCounters().at(0);

	// Setup device with this selection.
	device.setupInputFrame(inputs);

	// Read the values multiple times and write them to the console.
	const boost::posix_time::ptime startTime =
		boost::posix_time::second_clock::local_time();
	while (1) {
		// get local time
		boost::posix_time::ptime currentTime =
			boost::posix_time::second_clock::local_time();
		if ((currentTime - startTime).seconds() >= 20) {
			break;
		}

		// Read all inputs into the instance of InputFrame.
		InputFrame inFrame = device.readFrame();

		// Write results to the console.
		cout
			<< currentTime << ", "
			<< "DigitalPort #0: " << inFrame.getDigitalPort(0) << ", "
			<< "DigitalPort #1: " << inFrame.getDigitalPort(1) << ", "
			<< "SingleEnded #0: " << inFrame.getSingleEnded(0) << " V, "
			<< "SingleEnded #1: " << inFrame.getSingleEnded(1) << " V, "
			<< "Differential #1: " << inFrame.getDifferential(1) << " V, "
			<< "Counter #0: " << inFrame.getCounter(0) << endl;
	}

	// Write to analog out #1 and digital out #2 in one call.
	// Prepare and set output selection.
	OutputVector outputs;
	outputs
		<< device.getDigitalPorts().at(2)
		<< device.getAnalogOutputs().at(1);

	// Prepare device ...
	device.setupOutputFrame(outputs);

	// Prepare digital port #2 as output.
	device.getDigitalPorts().at(2).setOutputEnableMask(0xf);

	// Create instance of OutputFrame. There's no direct construction, as this
	// instance may vary between different device types.
	OutputFrame outFrame = device.createOutputFrame();

	// Write it to hardware, modify contents in every looping.
	for (int i = 0; i < 100; ++i) {
		outFrame.setAnalogOutput(1, (float)(3 * sin((float)i / 100.f)));
		outFrame.setDigitalPort(2, i);

		device.writeFrame(outFrame);
	}

	// Reset to power up defaults.
	device.resetDevice();
}

/**
 * Do frame IO (read/write multiple I/O in single calls).
 */
void frameIoTestStick(Device &device) {
	// Prepare and fill the list of inputs to read.
	InputVector inputs;
	inputs
		<< device.getSingleEndedInputs().at(0)
		<< device.getSingleEndedInputs().at(1)
		<< device.getDigitalPorts().at(0)
		<< device.getCounters().at(0);

	// Setup device with this selection.
	device.setupInputFrame(inputs);

	// Read the values multiple times and write them to the console.
	for (int i = 0; i < 100; ++i) {
		// Read all inputs into the instance of InputFrame.
		InputFrame inFrame = device.readFrame();

		// Write results to the console.
		cout
			<< "DigitalPort #0: " << inFrame.getDigitalPort(0) << ", "
			<< "SingleEnded #0: " << inFrame.getSingleEnded(0) << " V, "
			<< "SingleEnded #1: " << inFrame.getSingleEnded(1) << " V, "
			<< "Counter #0: " << inFrame.getCounter(0) << endl;
	}

	// Reset to power up defaults.
	device.resetDevice();
}

/**
 * Application entry point.
 */
int main() {
	// Nearly all methods can raise exceptions. For this example, the error handling is done at this single point.
	try {
		// Search for devices ...
		DeviceVector devices = LibraryInterface::enumerate(DeviceType::All);

		// If at least one has been found, use the first one ...
		if (!devices.empty()) {
			Device device = devices[0];

			// Open device, nothing can be done without doing this.
			device.open();

			// *** DIGITAL PORTS AND ANALOG OUTPUTS ARE MODIFIED DURING THIS ***
			// *** SECTION, SO DO NOT CONNECT ANY PERIPHERALS WITHOUT STUDYING ***
			// *** THE ACTIONS DONE HERE, OTHERWISE YOUR PERIPHERALS MAY BE DAMAGED !!! ***
			if (device.getDeviceType() == DeviceType::CeboLC)
				frameIoTestLC(device);
			else if (device.getDeviceType() == DeviceType::CeboStick)
				frameIoTestStick(device);
			else
				throw std::runtime_error("No supported device detected.");

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			device.close();
		}
	} // These exceptions are currently used to report different problems ...
	catch (const std::invalid_argument &e) { cout << "Invalid argument exception raised: " << e.what() << endl; }
	catch (const std::out_of_range &e) { cout << "Out of range exception raised: " << e.what() << endl; }
	catch (const std::logic_error &e) { cout << "Logic error exception raised: " << e.what() << endl; }
	catch (const std::runtime_error &e) { cout << "Runtime/IO error exception raised: " << e.what() << endl; }
	catch (const std::exception &e) { cout << "Unknown exception raised: " << e.what() << endl; }

	return 0;
}
